#include "uwbgl_Intersect2.h"

bool UWB_IntersectBoxBox( vec3 box1_min, vec3 box1_max, vec3 box2_min, vec3 box2_max) {
  // an intersection only exists if the boxes have some sort of overlap
  // along all three axes. Just test each axis individually
  bool xOutside = box1_min.x > box2_max.x || box1_max.x < box2_min.x;
  if(xOutside)
    return false;

  bool yOutside = box1_min.y > box2_max.y || box1_max.y < box2_min.y;
  if(yOutside)
    return false;

  bool zOutside = box1_min.z > box2_max.z || box1_max.z < box2_min.z;
  if(zOutside)
    return false;

   return true;
}

/////////////////////////////////////

UWB_BoundingBox::UWB_BoundingBox()
: m_min(0,0,0)
, m_max(0,0,0)
{
}

UWB_BoundingBox::UWB_BoundingBox(vec3 corner1, vec3 corner2)
{
  SetCorners(corner1, corner2);
}

UWB_BoundingBox::UWB_BoundingBox(vec3 center, float width, float height, float depth)
{
  m_min.x = center.x - width / 2.0f;
  m_min.y = center.y - height / 2.0f;
  m_min.z = center.z - depth / 2.0f;
  m_max.x = center.x + width / 2.0f;
  m_max.y = center.y + height / 2.0f;
  m_max.z = center.z + depth / 2.0f;
}

void UWB_BoundingBox::SetCorners(vec3 corner1, vec3 corner2)
{
  // test each point to determine which as max/min
  m_min.x = corner1.x<corner2.x?corner1.x:corner2.x;
  m_min.y = corner1.y<corner2.y?corner1.y:corner2.y;
  m_min.z = corner1.z<corner2.z?corner1.z:corner2.z;

  m_max.x = corner1.x>corner2.x?corner1.x:corner2.x;
  m_max.y = corner1.y>corner2.y?corner1.y:corner2.y;
  m_max.z = corner1.z>corner2.z?corner1.z:corner2.z;
}

void UWB_BoundingBox::MoveCenterTo( vec3 new_center )
{
  vec3 old_center = GetCenter();
  vec3 delta = new_center - old_center;
  m_min += delta;
  m_max += delta;
}

bool UWB_BoundingBox::Intersects(const UWB_BoundingVolume& other) const
{
  volume_type vt = other.GetType();
  if(box == vt)
  {
    const UWB_BoundingBox* pOtherBox = static_cast<const UWB_BoundingBox*>(&other);
    return UWB_IntersectBoxBox(m_min, m_max, pOtherBox->GetMin(), pOtherBox->GetMax());
  }
  //If we get here, we are intersecting with an unknown type
  return false;
}

vec3 UWB_BoundingBox::GetCenter() const
{
  vec3 c = (m_min+m_max) / 2.0;
  return c;
}

float UWB_BoundingBox::Width() const
{
  return (m_max.x - m_min.x);
}

float UWB_BoundingBox::Height() const
{
  return (m_max.y - m_min.y);
}

float UWB_BoundingBox::Depth() const
{
  return (m_max.z - m_min.z);
}

bool UWB_BoundingBox::ContainsPoint(vec3 location) const
{
  // test along each axis to make sure the point is inside this volume
  bool xInside = location.x >= m_min.x && location.x <= m_max.x;
  if(!xInside)
    return false;

  bool yInside = location.y >= m_min.y && location.y <= m_max.y;
  if(!yInside)
    return false;

  bool zInside = location.z >= m_min.z && location.z <= m_max.z;
  if(!zInside)
    return false;

  return true;
}

void UWB_BoundingBox::Draw(UWB_DrawHelper* pDrawHelper) const
{
  if(pDrawHelper)
  {
    vec3 start = m_min;
    vec3 end = m_min;
    end.x = m_max.x;
    pDrawHelper->DrawLine(start, end);
    start = end;
    end.y = m_max.y;
    pDrawHelper->DrawLine(start, end);
    start = end;
    end.x = m_min.x;
    pDrawHelper->DrawLine(start, end);
    start = end;
    end.y = m_min.y;
    pDrawHelper->DrawLine(start, end);
  }
}

void UWB_BoundingBox::MakeInvalid()
{
  //make min > max
  m_min = vec3(1,1,1);
  m_max = vec3(-1,-1,-1);
}
bool UWB_BoundingBox::IsValid() const
{
  return (m_min.x <= m_max.x && m_min.y <= m_max.y && m_min.z <= m_max.z);
}

void UWB_BoundingBox::Add(const UWB_BoundingVolume* pBV)
{
  if(pBV)
  {
    volume_type vt = pBV->GetType();
    if(box == vt)
    {
      const UWB_BoundingBox* pBox = static_cast<const UWB_BoundingBox*>(pBV);
      Add(*pBox);
    }
  }
}

void UWB_BoundingBox::Add(const UWB_BoundingBox& box)
{
  if(!box.IsValid())
    return;

  if(!IsValid())
    *this = box;
  else
  {
    m_min.x = m_min.x<box.m_min.x? m_min.x:box.m_min.x;
    m_min.y = m_min.y<box.m_min.y? m_min.y:box.m_min.y;
    m_min.z = m_min.z<box.m_min.z? m_min.z:box.m_min.z;
    m_max.x = m_max.x>box.m_max.x? m_max.x:box.m_max.x;
    m_max.y = m_max.y>box.m_max.y? m_max.y:box.m_max.y;
    m_max.z = m_max.z>box.m_max.z? m_max.z:box.m_max.z;
  }
}
