// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "uwbgl_Common.h"
#include "uwbgl_Containers.h"
#include "uwbgl_DrawHelper2.h"
#include "uwbgl_XformInfo2.h"
#include "uwbgl_Primitive3.h"

/// A scene node class is used as a directed graph that contains primitives and transformations.
/// This can be used to create a parent child relationship between primitives.
class UWB_SceneNode
{
public:
  /// Default constructor
  /// \param name Each scenenode has a name. This can be usefull for debugging purposes
  UWB_SceneNode(const wchar_t* name=L"SceneNode");
  virtual ~UWB_SceneNode();

  /// Draw this node and all of it's contained children
  /// \param lod level of detail to use for drawing the primitives
  void Draw( eLevelOfDetail lod, UWB_DrawHelper& draw_helper ) const;

  /// Set the primitive that this node contains. If this node already contains a primitive,
  /// the current one is destroyed and replaced by the new primitive.
  /// This class controls the lifetime of the passed in primitive.
  /// \param pPrimitive the object that this node will contain. This object
  ///        must be created on the heap.
  void SetPrimitive(UWB_Primitive* pPrimitive);

  /// Get the primitive that this node contains.
  /// If no primitive is contained in this node, return NULL.
  UWB_Primitive* GetPrimitive();
  /// Get the primitive that this node contains.
  /// If no primitive is contained in this node, return NULL.
  const UWB_Primitive* GetPrimitive() const;

  /// Add a child scenenode to this node. A SceneNode can have an
  /// unlimited number of child nodes.
  /// \param pChild the scenenode to add. This node must be created on the heap
  ///        and it must not have any existing parent node
  void InsertChildNode(UWB_SceneNode* pChild);

  /// Return the number of top level child SceneNodes in this SceneNode. This does not
  /// include nodes that may be children of any nodes in m_child_nodes.
  int NumChildren() const;
  /// Get a child node at a given index. If the index is out of the range of children,
  /// return NULL
  UWB_SceneNode* GetChildNode(int index);
  /// Get a child node at a given index. If the index is out of the range of children,
  /// return NULL
  const UWB_SceneNode* GetChildNode(int index) const;

  void MoveNodeByVelocity( float elapsed_seconds );
  void SetVelocity(vec3 v);
  vec3 GetVelocity() const;

  UWB_XformInfo GetXformInfo() const;
  ///
  /// Set the transformation for this scene node.
  void SetXformInfo(const UWB_XformInfo& xform);

  ///
  /// Turn on/off visibility of the transformation pivot when drawing
  void SetPivotVisible( bool on );
  ///
  /// Is the transformation's pivot drawn
  bool IsPivotVisible() const;

  void SetName(const wchar_t* name) { m_Name = name;}
  virtual const wchar_t* GetName() const { return m_Name.c_str(); }

  // Get Simple Bounding Box that encloses all objects in SceneNode
  UWB_BoundingBox GetBounds( UWB_DrawHelper& helper, bool bDraw = false ) const;

  // Get Bounding Box in world coordinates for specific node
  bool GetNodeBounds( const UWB_SceneNode* pSearchNode,
                      UWB_BoundingBox& box,
                      UWB_DrawHelper& helper,
                      bool bDraw = false ) const;

protected:
  bool GetNodeBoundsHelper( const UWB_SceneNode* pSearchNode,
                            UWB_BoundingBox& box,
                            UWB_DrawHelper& helper,
                            int level,
                            bool bDraw) const;

  /// A node may or may not contain a primitive
  /// Each primitive item can only have one node parent.
  /// The node ALWAYS controls the lifetime of it's internal primitive.
  /// This means all primitive must be declared on the heap!!
  UWB_Primitive* m_pPrimitive;

  /// A node can contain a list of child nodes. These child nodes must be declared
  /// on the heap.
  UWB_PointerArray<UWB_SceneNode*> m_child_nodes;
  vec3 m_velocity;
  UWB_XformInfo m_xform;
  bool m_bPivotVisible;
  std::wstring m_Name;
};
